<%--
 * Component: iface
 *
 * Description:
 *   HTTP API command to create DICOM media
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 *
--%><%@
	page import="
			java.sql.*,
			java.util.*,
			java.util.regex.*,
			imagemed.dicom.*,
			imagemed.dotcom.*,
			imagemed.exception.*,
			imagemed.imsession.*,
			imagemed.jsputil.*,
			imagemed.types.flag.IMTypeur,
			imagemed.util.*"

%><%!

	static String getOptionValue( String paramValue, UserPreferences.Option[] options ) {
		if ( paramValue == null ) {
			return null;
		}
		
		for ( int i = 0; i < options.length; i++ ) {
			if ( paramValue.toLowerCase().equals( options[ i ].getValue().toLowerCase() ) ) {
				return options[ i ].getValue();
			}
		}

		return null;
	}

%><%

	IMSession is = new IMSession( request, response, out );
	is.response.setContentType( "text/plain" );

	if ( !IfaceToolbox.isRegOK( is ) ) {
		IfaceToolbox.printError( is, "Has no registration info for this IP" );
		return;
	}

	boolean hasForwardRight = is.hasRight( IMTypeur.FORWARD_EXPORT_RIGHT );
	boolean hasBurnLicense = is.tabbean.hasLicense(is, "8");

	if ( !hasBurnLicense || !hasForwardRight ) {
		IfaceToolbox.printError( is, "You don't have the right to perform this operation" );
		return;
	}

	String parOper = request.getParameter( "cOper" );
	if ( "start".equals( parOper ) ) {
		String mediaType = request.getParameter( "cMediaType" );
		String compression = request.getParameter( "cCompression" );
		boolean addViewer = "true".equals( request.getParameter( "cAddViewer" ) );

		// Reports are included by default only if report series/objects are selected to the study (or whole study was selected).
		DicomMediaCreator.IncludeReports includeReports = DicomMediaCreator.IncludeReports.AUTO;

		// The new policy is to use affirmative (positive) parameter names
		if ( "false".equals( request.getParameter( "cIncludeReport" ) ) ) {
			includeReports = DicomMediaCreator.IncludeReports.FALSE;
		}
		else if ( "true".equals( request.getParameter( "cIncludeReport" ) ) ) {
			includeReports = DicomMediaCreator.IncludeReports.TRUE;
		}
		
		String profileUser = request.getParameter( "cUser" );

		String[] anonymTags = request.getParameterValues( "cOverwriteTag" );

		String optMediaType = getOptionValue( mediaType, UserPreferences.SELECT_MEDIA_TYPE.getOptions() );
		String optCompression = getOptionValue( compression, UserPreferences.SELECT_MEDIA_COMPRESSION.getOptions() );
		
		if ( optMediaType == null ) {
			IfaceToolbox.printError( is, "Unrecognized Media Type" );
			return;
		}
		
		if ( optCompression == null ) {
			IfaceToolbox.printError( is, "Unrecognized Compression" );
			return;
		}

		TreeMap<String, DicomMediaCreator.StudySubSet> studySet = new TreeMap<String, DicomMediaCreator.StudySubSet>();
		Vector<String> warnings = new Vector<String>();

		// true indicates that any of the cStudy/cSeries/cObject parameters were provided 
		boolean hasSelectorParameter = false;

		{
			String[] parStudies = request.getParameterValues( "cStudy" );
			if ( parStudies != null ) {
				for ( String study : parStudies ) {
					if ( !studySet.containsKey( study ) ) {
						studySet.put( study, new DicomMediaCreator.StudySubSet() );
					}
				}
				hasSelectorParameter = true;
			}
		}
		
		{
			String[] parSeries = request.getParameterValues( "cSeries" );
			if ( parSeries != null ) {
				for ( String studySer : parSeries ) {
					try {
						int p = studySer.indexOf( "/" );
						String study = studySer.substring( 0, p );
						String series = studySer.substring( p + 1 );
						
						DicomMediaCreator.StudySubSet subSet = studySet.get( study );
						if ( subSet == null ) {
							subSet = new DicomMediaCreator.StudySubSet();
							subSet.addSeries( series );
							studySet.put( study, subSet );
						}
						else if ( subSet.includeEntireStudy() ) {
							// entire study already included - skip
							warnings.add( "Entire study already included in study list, ignoring cSeries parameter: " + studySer );
							continue;
						}
						else {
							subSet.addSeries( series );
						}
					}
					catch ( Throwable th ) {
						warnings.add( "Invalid cSeries parameter: " + studySer );
					}
				}
				hasSelectorParameter = true;
			}
		}

		{
			String[] parObjects = request.getParameterValues( "cObject" );
			if ( parObjects != null ) {
				for ( String studyObj : parObjects ) {
					try {
						int p = studyObj.indexOf( "/" );
						String study = studyObj.substring( 0, p );
						String object = studyObj.substring( p + 1 );
						
						DicomMediaCreator.StudySubSet subSet = studySet.get( study );
						if ( subSet == null ) {
							subSet = new DicomMediaCreator.StudySubSet();
							subSet.addObject( object );
							studySet.put( study, subSet );
						}
						else if ( subSet.includeEntireStudy() ) {
							// entire study already included - skip
							warnings.add( "Entire study already included in study list, ignoring cObject parameter: " + studyObj );
							continue;
						}
						else {
							subSet.addObject( object );
						}
					}
					catch ( Throwable th ) {
						warnings.add( "Invalid cObject parameter: " + studyObj );
					}
				}
				hasSelectorParameter = true;
			}
		}
		
		if ( !hasSelectorParameter ) {
			try {
				StudyFinder sf = new StudyFinder( is, true, false );
				Vector<String> studies = sf.collectStudyList( false, true );
				if ( studies != null ) {
					for ( String study : studies ) {
						studySet.put( study, new DicomMediaCreator.StudySubSet() );
					}
				}
			}
			catch ( IMIfaceException ifEx ) {
				is.out.println( ifEx.getFormattedMessage() );
				return;
			}
		}
		else {
			DicomMediaCreator.check( studySet, warnings );
		}
		
		if ( studySet.isEmpty() ) {
			IfaceToolbox.printError( is, "No study found" );
			return;
		}

		String coercionRules = null;
		if ( anonymTags != null ) {
			Pattern tagPattern = Pattern.compile( "^(\\p{XDigit}{4},\\p{XDigit}{4}):(.*)$" );

			StringBuilder sb = new StringBuilder( 256 );
			for ( String tag : anonymTags ) {
				Matcher m = tagPattern.matcher( tag );
				if ( !m.matches() ) {
					warnings.add( "Value of cOverwriteTag parameter could not be parsed (" + tag + ")" );
				}
				sb.append( "(" ).append( m.group( 1 ) );
				sb.append( ")=\"" );
				sb.append( Helper.replace( m.group( 2 ), "\"", "\\\"" ) );
				sb.append( "\"\n" );
			}

			if ( sb.length() > 0 ) {
				coercionRules = sb.toString();
			}
		}
		
		
		try {
			DicomMediaCreator mediaCreator = new DicomMediaCreator( is.regID, studySet, optMediaType, optCompression, addViewer, includeReports );
			
			if ( profileUser != null ) {
				mediaCreator.setProfileUser( profileUser );
			}
			else {
				// Pick up system default profile user if configured
				DualPropsConfigReader burnCfg = new DualPropsConfigReader( IMEnvBase.getEnv( "CONF" ) + "/cdburn.conf", IMEnvBase.getEnv( "CASES_HOME" ) + "/cfg/cdburn.conf" );
				IMProperties burnProps = burnCfg.getProperties();
				String defUser = (burnProps) == null ? DicomSrvToolbox.PROFILE_USER_SELF : burnCfg.getProperty( "profileuser", DicomSrvToolbox.PROFILE_USER_SELF );  
			
				if ( ! defUser.equals( DicomSrvToolbox.PROFILE_USER_SELF ) ) {
					mediaCreator.setProfileUser( defUser );
				}
			}

			if ( coercionRules != null ) {
				mediaCreator.setCoercionRules( coercionRules );
			}
			mediaCreator.start();
		
			String id = mediaCreator.getId();
			
			is.out.println( "#USERID: " + is.regID );
			is.out.println( "#RESULT: OK" );
			is.out.println( "#ID:" + id );

			if ( !warnings.isEmpty() ) {
				for ( String msg : warnings ) {
					is.out.println( "#WARNING: " + msg );
				}
			}
			
			return;
		}
		catch ( Exception ex ) {
			IMIfaceException imi = new IMIfaceException( is, ex.getMessage() );
			is.out.print( imi.getFormattedMessage() );
			return;
		}
	}
	else if ( "delete".equals( parOper ) ) {
		String id = request.getParameter( "cId" );
		if ( id == null ) {
			IfaceToolbox.printError( is, "Missing parameter: media creation process ID" );
			return;
		}

		DicomMediaSession dms = new DicomMediaSession( id );
		if ( dms.doesSessionExist() ) {
			dms.delete( is.regID );
		}
		IfaceToolbox.printOK( is );
		return;
	}
	else if ( "status".equals( parOper ) ) {
		String id = request.getParameter( "cId" );
		if ( id == null ) {
			IfaceToolbox.printError( is, "Missing parameter: media creation process ID" );
			return;
		}

		DicomMediaSession dms = new DicomMediaSession( id );
		if ( !dms.doesSessionExist() ) {
			IfaceToolbox.printError( is, "No such media creation process" );
			return;
		}
		
		if ( dms.isCompleted() ) {
			DicomMediaSession.Result result = dms.getResult();
			String fileName = dms.getFileName();
			if ( result.isOK() ) {
				is.response.setContentType( "text/plain" );
				is.out.println( "#USERID: " + is.regID );
				is.out.println( "#RESULT: DONE" );
				is.out.println( "#URL: " + is.httpx_host + "/cd/getiso.cgi/" + fileName );
				return;
			}
			else {
				IfaceToolbox.printError( is, result.getErrorMessage() );
				return;
			}
		}
		else {
			DicomMediaSession.ProgressInfo progressInfo = dms.calcProgress();
			is.response.setContentType( "text/plain" );
			is.out.println( "#USERID: " + is.regID );
			is.out.println( "#RESULT: IN-PROGRESS" );
			is.out.println( "#PROGRESS: " + ( (int) progressInfo.getTotalPercent() ) );
			return;
		}
	}
	else {
		IfaceToolbox.printError( is, "Unrecognized operation" );
		return;
	}


%>