<%--
 * Component: iface
 *
 * Description:
 *  WEB interface for editing study data
 *
 *  @param cStudy       Study Instance UID (exclusive with cOrder)
 *  @param cMode        Hard or Soft edit (default is soft)
 *  @param qe_XXXX				 value of the XXXX field ('XXXX' is the fields COLID in Dcfields)
 *
 * Last Updated     : $Author$
 * Update date/time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 *
--%><%@
	page import="java.util.*,java.sql.*, imagemed.imsession.*,imagemed.jsputil.*,
			imagemed.util.*,imagemed.types.string.*,imagemed.types.*,
			imagemed.tables.*, imagemed.imtables.*, imagemed.exception.*,imagemed.dcprocessing.EditStudy"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%!

	static IMTypePN typePN = (IMTypePN) IMType.getTypeFor("PN");

	static HashSet<String> m_exceptFields;
	static {
		m_exceptFields = new HashSet<String>();
		m_exceptFields.add("FLDR");
		m_exceptFields.add("ICD9");
		m_exceptFields.add("CPT4");
		m_exceptFields.add("PDSC");
		m_exceptFields.add("PRID");
		m_exceptFields.add("PSYU");
		m_exceptFields.add("REAS");
		m_exceptFields.add("RQAD");
		m_exceptFields.add("RQEM");
		m_exceptFields.add("RQFN");
		m_exceptFields.add("RQPG");
		m_exceptFields.add("RQPN");
		m_exceptFields.add("RQPH");
		m_exceptFields.add("SCHD");
		m_exceptFields.add("SPSI");
		m_exceptFields.add("SCSD");
		m_exceptFields.add("STNM");
		m_exceptFields.add("SCHD");
		m_exceptFields.add("SONR");
		
		try {
			ResultSet objColids = IMEnv.getDB().createStatement().executeQuery("select COLID from Dctables where TABLENAME='Dcobject'");
			while (objColids.next()) {
				m_exceptFields.add(objColids.getString("COLID"));
			}
		}
		catch (SQLException se) {
			throw new RuntimeException(Helper.throwable2String(se));
		}
	}

	/**
	* Detects edit mode from request and config settings, and fills
	* editModePrp:
	*
	* Mode:           "Soft" or "Hard" edit
	* CanOverride:    "true" means user can override Mode
	*
	* @param is              session stuff
	* @param editModePrp     output parameter for holding edit mode settings
	*/
	static void getEditMode(IMSession is, Properties editModePrp) {
		IMProperties editModeCfg = new IMProperties();
		editModeCfg.recRead(IMEnvBase.getEnv("IFACE_VAR") + "/edit.cfg");

		String mode = editModeCfg.getProperty("EditMode","UserSoft");
		if ("Soft".equals(mode) || "Hard".equals(mode)) {
			editModePrp.setProperty("Mode", mode);
			return;
		}

		editModePrp.setProperty("CanOverride", "true");
		String parMode = is.request.getParameter("cMode");
		if (parMode != null) {
			editModePrp.setProperty("Mode", parMode);
		}
		else if ("UserHard".equals(mode)) {
			editModePrp.setProperty("Mode", "Hard");
		}
		else {
			editModePrp.setProperty("Mode", "Soft");
		}

		return;
	}

%><%

	IMSession is = new IMSession(request, response, out);
	if (!IfaceToolbox.isRegOK(is)) {
		IfaceToolbox.printError(is, "Has no registration info for this IP.");
		return;
	}

	String parStudy = request.getParameter("cStudy");

	if (parStudy == null || parStudy.length() == 0) {
		IfaceToolbox.printError(is, "Required parameter 'cStudy' is missing.");
		return;
	}

	Syslog slog = new Syslog("edit.jsp", 0 , IMEnvBase.getEnv("LOG_OPER_FACILITY"));
	Defs casesDefs = new Defs(is, null);

	boolean hasEditRight = false;
	if ( is.regRights != null ) {
		hasEditRight       = (-1 != is.regRights.indexOf('E'));
	}

	// proxyRepURL_ needs regGroup to be set
	if (is.regGroup == null)
		is.regGroup = "Admin";
		
	// Check if the user has rights to edit studies and/or creating orders
	if ( ! hasEditRight ) {
		OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "edit" );
		logEntry.setField( OperLogEntry.FIELD.USER, casesDefs.getUser() );
		logEntry.setField( OperLogEntry.FIELD.STUDY, parStudy );
		logEntry.setField( OperLogEntry.FIELD.MESSAGE, "no rights" );
		logEntry.log( slog, Syslog.LOG_ERR );

//		slog.syslog(Syslog.LOG_ERR, "ERROR STUDY edit " + casesDefs.getUser() + " - " + parStudy + " - - - - - - - - - - no_rights");
		IfaceToolbox.printError(is, "Has no access rights to edit studies.");
		return;
	}

	Properties lockHeaders = new Properties();
	String lockID = IMLock.lockStudy(is, parStudy, "Lock", lockHeaders);
	if (lockID == null) {
		String ownerMsg = "";
		if (lockHeaders.getProperty("IMLockOwner") != null) {
			ownerMsg = " Study is locked by " + lockHeaders.getProperty("IMLockOwner");
		}

//		for (Enumeration e = lockHeaders.propertyNames(); e.hasMoreElements(); ) {
//			String lh = (String) e.nextElement();
//			out.println("LOCK header: " + lh + " := " + lockHeaders.getProperty(lh));
//		}

		IfaceToolbox.printError(is, "Locking the study failed!" + ownerMsg);
		return;
	}

	try {
		is.sessionVariables.setProperty("IMLockID", lockID);
		is.sessionVariables.setProperty("httpIface", "true");

		/********************
		*   Save changes   *
		********************/
		
		ResultSet cols  = IMEnv.getDB().createStatement().executeQuery( "SELECT DISTINCT COLID FROM " + IMEnvBase.getEnv("FIELDSTBL") +
				" natural join Dctables WHERE (DICOMTAG<>'' OR CUSTOMFIELD<>'') AND MODFLAG in ('R','') AND TABLENAME='Dcstudy'" +
				"ORDER BY UINAME" );
		TreeSet<String> fieldList = new TreeSet<String>();
		while (cols.next()) {
			String fld = cols.getString( "COLID" );
			if ( m_exceptFields.contains( fld ) ) {
				// Editing this field is not supported
				continue;
			}
			if ( request.getParameter( "qe_" + fld ) == null ) {
				// No request parameter for field;
				continue;
			}
			
			fieldList.add(fld);
		}

		Vector<String> warnColids = new Vector<String>();
		for ( Enumeration<String> e = request.getParameterNames(); e.hasMoreElements(); ) {
			String paramName = e.nextElement();
			if ( paramName.startsWith( "qe_" ) ) {
				String colid = paramName.substring( 3 );
				if ( ! fieldList.contains( colid ) ) {
					warnColids.add( "Skipping unrecognized/unsupported field: " + colid );
				}
			}
		}
		
		String[] fields = fieldList.toArray(new String[0]);

		String SQLtables[]     = { "Dcstudy", "Dcobject" , "" };
		IMFieldManager fm = new IMFieldManager("Dcfields", "Dctypes", SQLtables);

		TableDescriptor tdesc       = new TableDescriptor();
		WorklistTableParams tparams = new WorklistTableParams(is.imRequest, casesDefs.getFilterUser());
		tparams.setDefaultFields(fields);

		Properties filters = new Properties();
		filters.setProperty("SYUI",parStudy);
		tparams.setDefaultFilters(filters);

		tdesc.setTableParams(tparams);
		tdesc.setFieldManager(fm);
		WorklistResult result = new WorklistResult(is, tdesc);
		result.setGlobal(true);

		if (!result.next()) {
			throw new IMException("No such study.");
		}

		String logPatient = result.getValue("PANM");
		String logModality = result.getValue("MODA");

		logPatient = Syslog.logValue(logPatient);
		logModality = (null == logModality || "".equals(logModality)) ? "-" : Helper.replace(logModality," ","_");

		Properties logPrp = new Properties();
		logPrp.setProperty("LOG_PATIENT",  logPatient);
		logPrp.setProperty("LOG_MODALITY", logModality);

		Properties modePrp = new Properties();
		getEditMode(is, modePrp);
		EditStudy.EditMode editMode = "Hard".equals(modePrp.getProperty("Mode", "Soft"))
				? EditStudy.EditMode.HARD : EditStudy.EditMode.SOFT;

		Properties rspHeaders = new Properties();
		EditStudy editStudy = new EditStudy(parStudy, null, slog, editMode);
		if (null == editStudy.edit(fields, is, rspHeaders, logPrp)) {
			throw new IMException("An error occured while editing study");
		}

		IfaceToolbox.printOK( is, warnColids );
	}
	catch (IMException imEx) {
		String errMsg = imEx.getMessage();
		if (errMsg == null) {
			errMsg = "An error occured while saving data";
		}
		IfaceToolbox.printError(is, errMsg);
	}
	catch (Throwable th) {
		IfaceToolbox.printError(is, "An error occured while saving data");
	}

	lockHeaders.clear();
	IMLock.lockStudy(is, parStudy, "Unlock", lockHeaders);
%>
