<%--
 ** Component: iface
 **
 ** Description:
 **   interface to initiate folder related operations
 **
 ** Parameters:
 **
 **		common Parameters:
 **				cOper		: operation (one of "addToFolder", "countFolder", "createFolder", "deleteFolder", 
 **																		"getAllFolders", "getAllUsers", "getFolders", "getFolderUsers",
 **																		"isStudyStatic", "listFolder", "removeFromFolder", "renameFolder")
 **		params for "addToFolder":
 **				cFolder			:	identifies the folder to which the study is to be added (like "rty/folder")
 **				cStudyUid		: identifies the study to be added (like "1.2.3.4")
 **				cStatically	: indicated whether a link or a copy should be made ("true" for a copy)
 **		returns			: if #RESULT iface header is "OK", #STYIUID iface header is also present 
 **									indicating the STYIUID of the link/copy (equal to cStudyUid if cStatically wasn't "true")
 **
 **		params for "countFolder":
 **				cFolder			:	identifies the folder in which the studies should be counted (like "rty/folder")
 **		returns			: if #RESULT iface header is "OK", #COUNT iface header is also present indicating the real result
 **
 **		params for "createFolder":
 **				cFolder			:	identifies the folder to create (like "rty/folder")
 **		returns			: nothing
 **
 **		params for "deleteFolder":
 **				cFolder			:	identifies the folder to delete (like "rty/folder")
 **		returns			: nothing
 **
 **		params for "getAllFolders":
 **				none...
 **		returns			: after headers a newLine separated list of (user/folder)s on server in "user" <TAB> "folder" format
 **
 **		params for "getAllUsers":
 **				none...
 **		returns			: after headers a newLine separated list of users
 **
 **		params for "getFolders":
 **				cUser				: the user who's folder should be listed
 **		returns			: after headers a newLine separated list of folders owned by the given user
 **
 **		params for "getFolderUsers":
 **				none...
 **		returns			: after headers a newLine separated list of users who own at least one folder
 **
 **		params for "isStudyStatic":
 **				cFolder			:	identifies the folder in which the study should be checked (like "rty/folder")
 **				cStudyUid		: identifies the study to be checked (like "1.2.3.4")
 **		returns			: if #RESULT iface header is "OK", #STATIC iface header is also present with the value 
 **									"true" if the study is static
 **
 **		params for "listFolder":
 **				cFolder			:	identifies the folder to list (like "rty/folder")
 **		returns			: after headers a newLine separated list of studies in the given folder
 **
 **		params for "removeFromFolder":
 **				cFolder			:	identifies the folder from which the study should be removed (like "rty/folder")
 **				cStudyUid		: identifies the study to be removed (like "1.2.3.4")
 **		returns			: nothing
 **
 **		params for "renameFolder":
 **				cFolder			:	identifies the folder to rename (like "rty/folder")
 **				cNewName		: the new name of the folder (like "folder2")
 **				cProxied		: used in communication between CWL server and it's children only
 **		returns			: nothing
 **
 ** $Author$
 ** $Date$
 **
 *
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
--%><%@
  page import="imagemed.jsputil.*, imagemed.imsession.*, java.util.*, java.io.*,java.sql.*,imagemed.util.*,imagemed.exception.*"

%><%@
  page errorPage ="../main/exceptionHandler.jsp"
%>
<%
	IMSession is = new IMSession(request, response, out);
  
	is.response.setContentType("text/plain");
	
	if (!IfaceToolbox.isRegOK(is)) {
		IfaceToolbox.printError(is, "Has no registration info for this IP.");
		return;
	}
%>
<%!private static Map<String, String[]> OPERMAP = new HashMap<String, String[]>();

	static {
		OPERMAP.put("addToFolder", new String[] {"cFolder", "cStudyUid", "cStatically"});
		OPERMAP.put("countFolder", new String[] {"cFolder"});
		OPERMAP.put("createFolder", new String[] {"cFolder"});
		OPERMAP.put("deleteFolder", new String[] {"cFolder"});
		OPERMAP.put("getAllFolders", new String[] {});
		OPERMAP.put("getAllUsers", new String[] {});
		OPERMAP.put("getFolders", new String[] {"cUser"});
		OPERMAP.put("getFolderUsers", new String[] {});
		OPERMAP.put("isStudyStatic", new String[] {"cFolder", "cStudyUid"});
		OPERMAP.put("listFolder", new String[] {"cFolder"});
		OPERMAP.put("removeFromFolder", new String[] {"cFolder", "cStudyUid"});
		OPERMAP.put("renameFolder", new String[] {"cFolder", "cNewName"});
	}

	/*
	 *	this method returns true if the user has admin rights or is identical to the given one
	 *	(for example he want's to alter his own folder or has admin right to alter someone else's...)
	 */
	private boolean hasRight(IMSession is, String user) throws IOException {
		if (IMAdminToolbox.bAdminRight(is)) {
			return true;
		}
		if (user != null && user.equals(is.regID)) {
			return true;
		}
  	IfaceToolbox.printError(is, "You don't have the right to perform this operation!");
		return false;
	}

	private void addToFolder(IMSession is) throws IOException {
		String cFolder = is.request.getParameter("cFolder");
		String cStudyUid = is.request.getParameter("cStudyUid");
		boolean cStatically = "true".equals(is.request.getParameter("cStatically"));
		String[] fldrParts = FolderToolbox.decomposeFolderName(cFolder);
		if (!hasRight(is, fldrParts[0])) {
			return;
		}
		String result = FolderToolbox.addToFolder(is, cFolder, cStudyUid, cStatically, null);
		IfaceToolbox.printOK(is);
		is.out.println("#STYIUID: " + result);
	}

	private void countFolder(IMSession is) throws IOException, SQLException {
		String cFolder = is.request.getParameter("cFolder");
		long size = FolderToolbox.countFolder(cFolder);
		IfaceToolbox.printOK(is);
		is.out.println("#COUNT: " + size);
	}

	private void createFolder(IMSession is) throws IOException, IMException {
		String cFolder = is.request.getParameter("cFolder");
		boolean acceptExisting = "true".equals( is.request.getParameter( "cAcceptExisting" ) );
		String[] fldrParts = FolderToolbox.decomposeFolderName(cFolder);
		if (!hasRight(is, fldrParts[0])) {
			return;
		}
		FolderToolbox.createFolder(is, cFolder, acceptExisting );
		IfaceToolbox.printOK(is);
	}

	private void deleteFolder(IMSession is) throws IOException, SQLException {
		String cFolder = is.request.getParameter("cFolder");
		String[] fldrParts = FolderToolbox.decomposeFolderName(cFolder);
		if (!hasRight(is, fldrParts[0])) {
			return;
		}
		String regID = is.regID;
		if (regID == null || is.regID.length() == 0) {
			is.regID = "admin";
		}
		FolderToolbox.deleteFolder(is, cFolder);
		is.regID = regID;
		IfaceToolbox.printOK(is);
	}

	private void getAllFolders(IMSession is) throws IOException, SQLException {
		if (!hasRight(is, null)) { // only for admins
			return;
		}
		List<String> folders = FolderToolbox.getAllFolders();
		IfaceToolbox.printOK(is);
		for (int i=0; i<folders.size(); i++) {
			String[] fldrParts = FolderToolbox.decomposeFolderName(folders.get(i));
			is.out.println(fldrParts[0]+"\t"+fldrParts[1]);
		}
	}

	private void getAllUsers(IMSession is) throws IOException, SQLException {
		if (!hasRight(is, null)) { // only for admins
			return;
		}
		List<String> users = FolderToolbox.getAllUsers();
		IfaceToolbox.printOK(is);
		for (int i=0; i<users.size(); i++) {
			is.out.println(users.get(i));
		}
	}

	private void getFolders(IMSession is) throws IOException {
		String cUser = is.request.getParameter("cUser");
		if (!hasRight(is, cUser)) {
			return;
		}
		List<String> folders = FolderToolbox.getFolders(cUser);
		IfaceToolbox.printOK(is);
		for (int i=0; i<folders.size(); i++) {
			String[] fldrParts = FolderToolbox.decomposeFolderName(folders.get(i));
			is.out.println(fldrParts[0]+"\t"+fldrParts[1]);
		}
	}

	private void getFolderUsers(IMSession is) throws IOException, SQLException {
		if (!hasRight(is, null)) { // only for admins
			return;
		}
		List<String> users = FolderToolbox.getFolderUsers();
		IfaceToolbox.printOK(is);
		for (int i=0; i<users.size(); i++) {
			is.out.println(users.get(i));
		}
	}

	private void isStudyStatic(IMSession is) throws IOException, SQLException, IMException {
		String cFolder = is.request.getParameter("cFolder");
		String cStudyUid = is.request.getParameter("cStudyUid");
		String[] fldrParts = FolderToolbox.decomposeFolderName(cFolder);
		if (!hasRight(is, fldrParts[0])) {
			return;
		}
		boolean result = cStudyUid.equals( StudyToolbox.getSourceStudyDB( cStudyUid ) );
		IfaceToolbox.printOK(is);
		is.out.println("#STATIC: " + result);
	}

	private void listFolder(IMSession is) throws IOException, SQLException {
		String cFolder = is.request.getParameter("cFolder");
		String[] fldrParts = FolderToolbox.decomposeFolderName(cFolder);
		if (!hasRight(is, fldrParts[0])) {
			return;
		}
		List<String> studies = FolderToolbox.listFolder(cFolder);
		IfaceToolbox.printOK(is);
		for (int i=0; i<studies.size(); i++) {
			is.out.println(studies.get(i));
		}
	}

	private void removeFromFolder(IMSession is) throws IOException {
		String cFolder = is.request.getParameter("cFolder");
		String cStudyUid = is.request.getParameter("cStudyUid");
		String[] fldrParts = FolderToolbox.decomposeFolderName(cFolder);
		if (!hasRight(is, fldrParts[0])) {
			return;
		}
		String user = is.regID;
		if (user == null || user.length() == 0) {
			user = "admin";
		}
		FolderToolbox.removeFromFolder(is, cFolder, cStudyUid, user);
		IfaceToolbox.printOK(is);
	}

	private void renameFolder(IMSession is) throws IOException, SQLException {
		String cFolder = is.request.getParameter("cFolder");
		String cNewFolder = is.request.getParameter("cNewName");
		String[] fldrParts = FolderToolbox.decomposeFolderName(cFolder);
		if (!hasRight(is, fldrParts[0])) {
			IfaceToolbox.printError(is, "You don't have the rights to complete this operation.");
			return;
		}
		try {
			FolderToolbox.renameFolder(is, cFolder, cNewFolder);
			IfaceToolbox.printOK(is);
		}
		catch (IMException ime) {
			IfaceToolbox.printError(is, ime.getMessage());
		}
	}%><%
	String operStr = request.getParameter("cOper");
	if (operStr == null || operStr.length() == 0) {
		IfaceToolbox.printError(is, "Required parameter \"cOper\" is missing!");
		return;
	}
	if (!OPERMAP.containsKey(operStr)) {
		IfaceToolbox.printError(is, "Unknown operation: "+operStr);
		return;
	}
	String[] params = OPERMAP.get(operStr);
	for (int i=0; i<params.length; i++) {
		String parValue = request.getParameter(params[i]);
		if (parValue == null || parValue.length() == 0) {
			IfaceToolbox.printError(is, "Required parameter \""+params[i]+"\" for operation \""+operStr+"\" is missing!");
			return;
		}
	}
	try {
		if ("addToFolder".equals(operStr)) {
			addToFolder(is);
			return;
		}
		if ("countFolder".equals(operStr)) {
			countFolder(is);
			return;
		}
		if ("createFolder".equals(operStr)) {
			createFolder(is);
			return;
		}
		if ("deleteFolder".equals(operStr)) {
			deleteFolder(is);
			return;
		}
		if ("getAllFolders".equals(operStr)) {
			getAllFolders(is);
			return;
		}
		if ("getAllUsers".equals(operStr)) {
			getAllUsers(is);
			return;
		}
		if ("getFolders".equals(operStr)) {
			getFolders(is);
			return;
		}
		if ("getFolderUsers".equals(operStr)) {
			getFolderUsers(is);
			return;
		}
		if ("isStudyStatic".equals(operStr)) {
			isStudyStatic(is);
			return;
		}
		if ("listFolder".equals(operStr)) {
			listFolder(is);
			return;
		}
		if ("removeFromFolder".equals(operStr)) {
			removeFromFolder(is);
			return;
		}
		if ("renameFolder".equals(operStr)) {
			renameFolder(is);
			return;
		}
		IfaceToolbox.printError(is, "UnHandled operation: "+operStr);
		return;
	}
	catch (SQLException se) {
		IfaceToolbox.printError(is, "An SQLException occured during operation: "+se.getMessage());
		return;
	}
	catch (IOException ioe) {
		IfaceToolbox.printError(is, "An IOException occured during operation: "+ioe.getMessage());
		return;
	}
	catch (IMException ime) {
		IfaceToolbox.printError(is, "An IMException occured during operation: "+ime.getMessage());
		return;
	}
	catch (RuntimeException re) {
		IfaceToolbox.printError(is, "Error during operation: "+Helper.throwable2String(re));
		return;
	}
%>