<%--
 * Component: iface
 *
 * Description:
 *    Allow opening a study using Accession Number(s)
 * 
 * If there are more than one studies for an accession number, an error response will be returned.
 *
 * @params:
 *
 *   ACCN   accession number of the study to be opened.
 *          More than one accession number can be specified separated by semicolon (;)
 *   SYUI   study instance UID of the study to be opened
 *          More than one study can be specified separated by semicolon (;)
 *
 *   cMultipleMode    how to handle multiple matches on accession numbers
 *                    possible values are
 *                    normal (default):  return an error message
 *                    open            :  open all matching studies in the viewer
 *                    list            :  popup worklist with matching studies
 *
 *   cUser    user id
 *   cPasswd  password
 *
 *
 *   For internal use only 
 *   cAppend          true indicates to open studies in existing viewer session by keeping other studies open
 *   cViewSessionID   viewer session the studies to be appened to. 
 *   cOpenMan         true indicates to allow opening empty study lists and non-existing studies
 *                    and to bring up Manager Window in the viewer
 *
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 *
--%><%@
	page import="imagemed.tables.*, imagemed.imtables.*, imagemed.imsession.*, java.util.*,
			imagemed.util.Helper, imagemed.jsputil.*, imagemed.exception.*"
	
%><%!

	static final int MULTIPLE_MODE_NORMAL = 0;
	static final int MULTIPLE_MODE_OPEN = 1;
	static final int MULTIPLE_MODE_LIST = 2;

%><%
	IMSession is = new IMSession( request, response, out );
	String parAccno = request.getParameter( "ACCN" );
	String parStudies = request.getParameter( "SYUI" );
	String parPatient = request.getParameter( "PAID" );
	String parMultipleMode = request.getParameter( "cMultipleMode" );
	
	boolean isAppend = "true".equals( request.getParameter( "cAppend" ) );
	String viewSessionID =  request.getParameter( "cViewSessionID" );
	boolean needOpenMan = "true".equals( request.getParameter( "cOpenMan" ) );

	String sessionID =  request.getParameter( "sessionid" );

	int multipleMode = MULTIPLE_MODE_NORMAL;
	if ( "list".equals( parMultipleMode ) ) {
		multipleMode = MULTIPLE_MODE_LIST;
	}
	else if ( "open".equals( parMultipleMode ) ) {
		multipleMode = MULTIPLE_MODE_OPEN;
	}

	is.sessionVariables.setProperty("httpIface", "true");

	try {
		StudyFinder sf = new StudyFinder( is, true, false );
		Vector<String> studySet = sf.collectStudyList( multipleMode != MULTIPLE_MODE_OPEN, needOpenMan ); 

		if ( studySet.isEmpty() && ! needOpenMan ) {
			IfaceToolbox.printError( is, "No study found" );
			return;
		}

		StringBuffer passStudies = new StringBuffer();
		for ( Iterator<String> i = studySet.iterator(); i.hasNext(); ) {
			passStudies.append( "&cGr=" + i.next() );
		}

		if ( isAppend ) {
			passStudies.append( "&Append=true" );
		}
		if ( viewSessionID != null ) {
			passStudies.append( "&ViewSessionID=" + Helper.URLEncodeUTF8( viewSessionID ) );
		}
		
		if ( needOpenMan ) {
			StringBuilder openManParams = new StringBuilder( 256 );
			if ( parAccno != null ) {
				openManParams.append( "&ACCN=" ).append( Helper.URLEncodeUTF8( parAccno ) );
			}
			if ( parStudies != null ) {
				openManParams.append( "&SYUI=" ).append( Helper.URLEncodeUTF8( parStudies ) );
			}
			if ( parPatient != null ) {
				openManParams.append( "&PAID=" ).append( Helper.URLEncodeUTF8( parPatient ) );
			}

			passStudies.append( "&cOpenMan=" + Helper.URLEncodeUTF8( openManParams.substring( 1 ) ) );
		}
		
		String url = GetpbsToolbox.getURLforGetpbs( );
		String loc = is.httpx_host + url + "?" + passStudies.substring( 1 );
		if( sessionID != null ) {
			loc += "&sessionid=" + sessionID;
		}
		IfaceToolbox.loginAndRedirect( is, loc );

	}
	catch ( IMMultipleMatchException mmex ) {
		if ( multipleMode == MULTIPLE_MODE_LIST ) {
			// Popup worklist
			StringBuffer redirectParams = new StringBuffer();
			if ( is._sid != null ) {
				redirectParams.append( "&sessionid=" ).append( Helper.URLEncodeUTF8( is._sid ) );
			}
			if ( parAccno != null ) {
				redirectParams.append( "&qf_ACCN=" ).append( Helper.URLEncodeUTF8( parAccno ) );
			}
			if ( parStudies != null ) {
				redirectParams.append( "&qf_SYUI=" ).append( Helper.URLEncodeUTF8( parStudies ) );
			}
			if ( parPatient != null ) {
				redirectParams.append( "&qf_PAID=" ).append( Helper.URLEncodeUTF8( parPatient ) );
			}

			IfaceToolbox.loginAndRedirect( is, is.httpx_host + "/cases/index.jsp?" + redirectParams.substring( 1 ) );
			return;
		}

		response.setContentType( "text/plain" );
		is.out.println( mmex.getFormattedMessage() );
	}
	catch ( IMIfaceException ex ) {
		response.setContentType( "text/plain" );
		is.out.println( ex.getFormattedMessage() );
	}
	
	if ( true ) {
		// To avoid sending any other character on the connection after redirect
		return;
	}
%>