<%--
 * Component: iface
 *
 * WEB interface for importing reports from external systems (RIS).
 *
 *
 * $Author$
 * $Date$
 *
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
 *
 * Supported fields
 *
 * PatientID               PAID
 * PatientName             PANM
 * BirthDateTime           PDOB
 * Sex                     PSEX
 * Address                 PADD
 * PhoneNumbers            PAPH
 * Height                  -
 * Weight                  -
 * 
 * Study Description       DSCR
 * Reading Physician       PHRD
 * Referring Physician     PHRF
 * Study Date              SYDT
 *
 * PlacerOrderNumber       -
 * FillerOrderNumber       -
 * AccessionNumber         ACCN
 * Interpretation ID       INID
 * Modality                MODA
 * ObservationDateTime     IRDT
 * TranscriptionDateTime   ITDT
 * ApprovalDateTime        IADT
 * AuthorID                ITAI
 * AuthorName              ITAN
 * TranscriptionistID      ITPI
 * TranscriptionistName    ITPN
 * ApproverID              IAPI
 * ApproverName            IAPN
 * StudyIUID               SYUI
 * SRIUID                  SPUI
 * Reading Location ID     ILCI
 * Normality Status        INIS
 * BI-RADS Code            BRDR
 *
 *
 * Regular Parameters:
 *
 * Impression    cImpr
 * Observation   cText
 * MergeFlag     cMerge // true means merge the report into the first editable SR file of the study
 * Status        cStatus // { 50, 60, 80, 100 } default is 100
 *
 * cBack : redirect instead of writing to standard output
 *
--%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%@
	page import="imagemed.util.*, imagemed.imtables.*, imagemed.exception.*, imagemed.dotcom.DotcomConfig,
			imagemed.imsession.*, imagemed.jsputil.*, java.util.*, java.sql.*"

%><%-- Input Parameters --%><%
	IMSession is = new IMSession( request, response, out );

	String parBack = request.getParameter( "cBack" );

	Syslog slog = new Syslog( "importReport.jsp", 0 , IMEnvBase.getEnv( "LOG_OPER_FACILITY" ) );

%><%!

	static String fields[] = {
		"ACCN", // AccessionNumber
		"DSCR", // Description
		"MODA", // Modality
		"PADD", // Address
		"PAID", // PatientID
		"PANM", // PatientName
		"PAPH", // PhoneNumbers
		"PDOB", // BirthDateTime
		"PHRD", // Reading Physician
		"PHRF", // Referring Physician
		"PSEX", // Sex
		"SPUI", // SR SOP Instance UID
		"SYDT", // Study Date
		"SYUI", // StudyInstanceUID
		"IADT", // ApprovalDateTime
		"IAPI", // Approver ID
		"IAPN", // ApproverName
		"INID", // Interpretation ID
		"IRDT", // ObservationDateTime
		"ITAI", // Author ID
		"ITAN", // AuthorName
		"ITPI", // Transcriptionist ID
		"ITDT", // Transcription Date Time
		"ITPN", // Transcriptionist Name
		"ILCI", // Registered Reading Location ID
		"INIS", // Normality Status
		"BRDR"  // BI-RADS code
	};

	static String commonFields[] = {
		"ACCN", // AccessionNumber
		"DSCR", // Description
		"MODA", // Modality
		"PADD", // Address
		"PAID", // PatientID
		"PANM", // PatientName
		"PAPH", // PhoneNumbers
		"PDOB", // BirthDateTime
		"PHRD", // Reading Physician
		"PHRF", // Referring Physician
		"PSEX", // Sex
		"SYDT", // Study Date
	};

	// This is for setting report status when importing reports via HTTP interface
	// We may want to create and use IMTypeISTI instead
	static Properties REPORT_VALUES;
	static {
		// Report values
		REPORT_VALUES = new Properties();

		REPORT_VALUES.setProperty( "APPROVED", "APPROVED" );
		REPORT_VALUES.setProperty( "100", "APPROVED" );
		REPORT_VALUES.setProperty( "TRANSCRIBED", "TRANSCRIBED" );
		REPORT_VALUES.setProperty( "80", "TRANSCRIBED" );
		REPORT_VALUES.setProperty( "RECORDED", "RECORDED" );
		REPORT_VALUES.setProperty( "60", "RECORDED" );
		REPORT_VALUES.setProperty( "CREATED", "CREATED" );
		REPORT_VALUES.setProperty( "50", "CREATED" );
	}
	
	static String FINAL_STATUS = "APPROVED";

	static class UidPair {
		String m_study;
		String m_proposedStudy;
		
		UidPair() {
			m_study = null;
			m_proposedStudy = null;
		}
		
		void setStudy( String study ) {
			m_study = "".equals( study ) ? null : study;
		}
		void setProposedStudy( String proposedStudy ) {
			m_proposedStudy = "".equals( proposedStudy ) ? null : proposedStudy;
		}
		String getStudy() {
			return m_study;
		}
		String getProposedStudy() {
			return m_proposedStudy;
		}
	};

	
	/**
	* Creates mod file entry for a DICOM tag
	*
	* @param tag    create mod file for this DICOM tag (format is: 'GGGG,UUUU')
	* @param vr     value representation of the tag
	* @param value  value of the tag
	*
	* @return  the created mod file entry
	*/
	static String getModFileEntry( String tag, String vr, String value ) {
		if ( tag == null || vr == null || value == null )
			return "";

		return "(" + tag + ") " + vr + " [" + value + "]\n";
	}


	/**
	* Creates mod file entry/entries for a field
	*
	* @param field  create mod file entry/entries for this field
	* @param value  value of the field
	* @param fm     field manager used for processing field value.
	*
	* @return  the created mod file entry/entries
	*/
	static String createModFileEntry( String field, String value, IMFieldManager fm ) {
		String returnVal = "";
		try {
			String tag = fm.getDicomTag( field );
			int p = tag.indexOf( "_" );
			if ( p != -1 ) { // Date Time
				String dateTag = tag.substring( 0, p );
				String timeTag = tag.substring( p + 1 );
				int q = value.indexOf( "_" );
				if ( q == -1 ) {
					String davalue = imagemed.types.IMType.getTypeFor( "DA" ).canonize( value );
					returnVal = getModFileEntry( dateTag, "DA", davalue );
				}
				else {
					String dtvalue = imagemed.types.IMType.getTypeFor( "DA_TM" ).canonize( value );
					q = dtvalue.indexOf( "_" );
					returnVal = getModFileEntry( dateTag, "DA", dtvalue.substring( 0, q ) );
					returnVal = returnVal + getModFileEntry( timeTag, "TM", dtvalue.substring( q + 1 ) );
				}
			}
			else {
				String vr = fm.getVR( field );
				returnVal = getModFileEntry( tag, vr, value );
			}
		}
		catch ( Throwable th ) {
		}
		return returnVal;
	}

	static void addAuthorObserver( Report.Participants participants, String id, String name, String dateTime ) {
		if ( name != null &&  !"".equals( name ) ) {
			participants.setAuthor( id, name, false );
		}

		try {
			String cdateTime = imagemed.types.IMType.getTypeFor( "DA_TM" ).canonize( dateTime );
			if ( ! "".equals( cdateTime ) ) {
				int q = cdateTime.indexOf( "_" );
				participants.setContentTimestamp( cdateTime.substring( 0, q ), cdateTime.substring( q + 1 ), false );
			}
		}
		catch ( Throwable th ) {
		}
	}

	static void addApprover( Report.Participants participants, String id, String name, String dateTime ) {
		if ( name == null || "".equals( name ) ) {
			return;
		}
		
		String date = null;
		String time = null;

		try {
			String cdateTime = imagemed.types.IMType.getTypeFor( "DA_TM" ).canonize( dateTime );
			if ( ! "".equals( cdateTime ) ) {
				int q = cdateTime.indexOf( "_" );
				date = cdateTime.substring( 0, q );
				time = cdateTime.substring( q + 1 );
			}
		}
		catch ( Throwable th ) {
		}
		
		participants.setApprover( id, name, date, time );
	}
	
	static void addTranscriptionist( Report.Participants participants, String id, String name, String dateTime ) {
		if ( name == null || "".equals( name ) ) {
			return;
		}
		
		String date = null;
		String time = null;

		try {
			String cdateTime = imagemed.types.IMType.getTypeFor( "DA_TM" ).canonize( dateTime );
			if ( ! "".equals( cdateTime ) ) {
				int q = cdateTime.indexOf( "_" );
				date = cdateTime.substring( 0, q );
				time = cdateTime.substring( q + 1 );
			}
		}
		catch ( Throwable th ) {
		}
		
		participants.setTranscriber( id, name, date, time, true );
	}
	

	static UidPair findMatchingStudy( String uid, String accno, String pid ) throws IMException {
		UidPair uidPair = new UidPair();

		if ( uid != null ) {
			// Check if the provided study is an order and use Proposed Study Instance UID instead
			String query = "select MAINST, STYIUID, PSTYIUID from " + IMEnvBase.getEnv( "STUDYTBL" ) +
			" where STYIUID='" + Helper.sqlEncode( uid ) + "' AND Dcstudy_D='no' AND DERIVED!='shortcut'";
			
			try ( ResultSet rs = IMEnv.getDB().prepareStatement( query ).executeQuery() ) {
				if ( rs.next() ) {
					int mainst = rs.getInt( "MAINST" );
					uidPair.setStudy( rs.getString( "STYIUID" ) );
	
					if ( mainst < 0 ) {
						uidPair.setProposedStudy( rs.getString( "PSTYIUID" ) );
					}
				}
			}
			catch( SQLException sqlEx ) {
				throw new IMException( "An error occurred while looking up database for the study" );
			}
			
			return uidPair.getStudy() == null ? null : uidPair;
		}
			
		if ( accno == null ) {
			String studyUiName = IMBaseFieldManager.staticGetUiName( "SYUI" );
			String accnoUiName = IMBaseFieldManager.staticGetUiName( "ACCN" );

			throw new IMException( "No study identifier was specified. Either " + studyUiName + " (SYUI) or " +
					accnoUiName + " (ACCN) should be specified." );
		}

		StringBuilder query = new StringBuilder( 256 );

		query.append( "select STYIUID, PSTYIUID, (Dcstudy_D='yes') AS ISDELETED, (MAINST < 0) AS ISORDER from " );
		query.append( IMEnvBase.getEnv( "STUDYTBL" ) );
		query.append( " where FOLDER='' AND DERIVED != 'shortcut'" );
		
		String dbName = IMBaseFieldManager.staticGetDbName( "ACCN" );
		query.append( " AND " ).append( dbName ).append( "='" ).append( Helper.sqlEncode( accno ) ).append( "'" );
		if ( pid != null ) {
			String dbPidName = IMBaseFieldManager.staticGetDbName( "PAID" );
			query.append( " AND " ).append( dbPidName ).append( "='" ).append( Helper.sqlEncode( pid ) ).append( "'" );
		}

		try ( ResultSet rs = IMEnv.getDB().prepareStatement( query.toString() ).executeQuery() ) {
			boolean isOrder = false;
			while ( rs.next() ) {
				String dbStudy = rs.getString( "STYIUID" );
				String dbProposed = rs.getString( "PSTYIUID" );
				boolean dbIsDeleted = rs.getBoolean( "ISDELETED" );
				boolean dbIsOrder = rs.getBoolean( "ISORDER" );

				if ( uidPair.getStudy() != null ) {
					// There's other matching entry
					if ( dbIsDeleted ) {
						// Ignore this deleted entry
						continue;
					}
					
					if ( ! ( isOrder && dbIsOrder ) ) {
						// Any of the matching entries is a study (not order)
						throw new IMException( "Multiple studies match" );
					}

					// both entries are orders

					if ( dbProposed == null ) {
						// should never happen - as orders must have the dbProposedStudy filled
						throw new IMException( "Internal Error: Proposed Study UID is not set for order: %s\n", dbStudy );
					}

					if ( ! dbProposed.equals( uidPair.getProposedStudy() ) ) {
						// Matching orders belong to different order groups
						throw new IMException( "Multiple orders match" );
					}

					// Orders of the same group order
					continue;
				}
				
				// First matching entry or was only deleted matching - keep this entry
				uidPair.setStudy( dbStudy );
				uidPair.setProposedStudy( dbProposed );
				isOrder = dbIsOrder;
			}
		}
		catch( SQLException sqlEx ) {
			throw new IMException( "An error occurred while looking up database for the study" );
		}

		return uidPair.getStudy() == null ? null : uidPair;
	}


	/**
	 * Gets study identifiers from request parameters and looks up for study uid in the database
	 * Study identifier parameters (SYUI, ACCN, PAID) shall be taken from request.
	 * Legacy study identifier parameters (qe_SYUI, qe_ACCN) shall be taken from params (pre-processed request parameters).
	 *
	 * @param request  HTTP request having original request parameters
	 * @param params   pre-processed request parameters for legacy study identifiers
	 * 
	 * @return study (and proposed study) uids of the study found for the identifier parameters in the database
	 *
	 * @thwrows IMException if finding matching study is failed for any reason
	 */
	static UidPair getStudyIdentifiers( HttpServletRequest request, Properties params ) throws IMException {
		// Check study identifier parameters
		//    SYUI
		//    ACCN [ + PAID ]
		// Legacy identifiers
		//    qe_SYUI
		//    qe_ACCN

		String syui = request.getParameter( "SYUI" ); // 'SYUI' request parameter
		String accn = request.getParameter( "ACCN" ); // 'ACCN' request parameter
		String paid = request.getParameter( "PAID" ); // 'PAID' request parameter

		String qeStudy = params.getProperty( "SYUI" ); // 'qe_SYUI' request parameter
		String qeAccno = params.getProperty( "ACCN" ); // 'qe_ACCN' request parameter

		
		if ( syui == null ) {
			if ( qeStudy != null ) {
				syui = qeStudy;
			}
		}
		else if ( qeStudy != null ) {
			// Both qe_SYUI and SYUI was specified - values must be the same
			if ( !syui.equals( qeStudy ) ) {
				String studyUiName = IMBaseFieldManager.staticGetUiName( "SYUI" );
				throw new IMException( "Values for " + studyUiName + " (qe_SYUI and SYUI parameters) mismatch" );
			}
		}
		
		if ( "".equals( syui ) ) {
			syui = null;
		}

		if ( accn == null ) {
			if ( paid != null ) {
				String pidUiName = IMBaseFieldManager.staticGetUiName( "PAID" );
				String accnoUiName = IMBaseFieldManager.staticGetUiName( "ACCN" );
				throw new IMException( pidUiName + " (PAID) was specified without " + accnoUiName + " (ACCN) to identify the study" );
			}
		
			if ( qeAccno != null ) {
				// Fall back to old mechanism - matching on study uid or accession number
				accn = qeAccno;
			}
		}
		else if ( qeAccno == null ) {
			params.setProperty( "ACCN", accn );
		}
		else {
			// Both qe_ACCN and ACCN was specified - values must be the same
			if ( !accn.equals( qeAccno ) ) {
				String accnoUiName = IMBaseFieldManager.staticGetUiName( "ACCN" );
				throw new IMException( "Values for " + accnoUiName + " (qe_ACCN and ACCN parameters) mismatch" );
			}
		}
		
		if ( "".equals( accn ) ) {
			accn = null;
		}

		// 'PAID' is optional, checks only needed when set
		if ( paid != null ) {
			String qePid = params.getProperty( "PAID" ); // 'qe_PAID' request parameter
			if ( qePid == null ) {
				params.setProperty( "PAID", paid );
			}
			else if ( !paid.equals( qePid) ) {
				String pidUiName = IMBaseFieldManager.staticGetUiName( "PAID" );
				throw new IMException( "Values for " + pidUiName + " (qe_PAID and PAID parameters) mismatch" );
			}
		}

		
		UidPair uidPair = findMatchingStudy( syui, accn, paid );
		if ( uidPair == null ) {
			uidPair = new UidPair();
			uidPair.setStudy( syui ); // for the case study was provided but not on the worklist 
		}

		return uidPair;
	}

	
	/**
	* Generates Interpretation ID
	*
	* @return the generated interpretation ID
	*/
	synchronized static String createID() {
		long t = System.currentTimeMillis();
		String encT = Long.toString( t, Character.MAX_RADIX );
		return encT + "." + Helper.getCounter();
	}

%><%

	String errorMsg = null;

	try {

		// Check login
		if ( !IfaceToolbox.isRegOK( is ) ) {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "import" );
			logEntry.setField( OperLogEntry.FIELD.HOST, request.getRemoteAddr() );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "no rights" );
			logEntry.log( slog, Syslog.LOG_ERR );

//			slog.syslog(Syslog.LOG_ERR,
//					"ERROR STUDY import - - - - - " + request.getRemoteAddr() + " - - - - - - no_rights");
			throw new IMException( "Import Disabled." );
		}

		boolean hasReportRight = false;
		if ( is.regRights != null ) {
			hasReportRight = ( -1 != is.regRights.indexOf( 'R' ) );
		}

		// Check if the user has rights to create orders
		if ( !hasReportRight ) {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "import" );
			logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "no rights" );
			logEntry.log( slog, Syslog.LOG_ERR );

//			slog.syslog(Syslog.LOG_ERR,
//					"ERROR STUDY import " + is.regID + " - - - - - - - - - - - no_rights");
			throw new IMException( is.regID + " has no right to create reports." );
		}

		StringBuilder mod = new StringBuilder( 512 );

		String SQLtables[]     = { "Dcstudy", "Dcobject" , "Dcreport" };
		IMFieldManager fm = new IMFieldManager( "Dcfields", "Dctypes", SQLtables );
		// Order fields

		Properties params = IfaceToolbox.processParameters( request, fm, fields );

		// uidPair shall not be null
		UidPair uidPair = getStudyIdentifiers( request, params );
		
		// Process common (demography) fields
		for ( int i = 0 ; i < commonFields.length; i++ ) {
			String val = params.getProperty( commonFields[ i ] );
			if ( val != null )
				mod.append( createModFileEntry( commonFields[ i ], val, fm ) );
		}

		// Merge flag
		String merge = request.getParameter( "cMerge" );

		// Report fields
		String impressionText = request.getParameter( "cImpr" );
		String findingsText = request.getParameter( "cText" );

		String impressionFileName =
				IfaceToolbox.createTmpFile( request.getParameter( "cImpr" ), DotcomConfig.selfId() + "_import_impr" );
		String textFileName =
				IfaceToolbox.createTmpFile( request.getParameter( "cText" ), DotcomConfig.selfId() + "_import_text" );
		
		Properties reportFiles = new Properties();
		reportFiles.setProperty( impressionFileName, impressionText == null ? "" : impressionText );
		reportFiles.setProperty( textFileName, findingsText == null ? "" : findingsText );
		
		String status = request.getParameter( "cStatus" );
		if ( status == null )
			status = FINAL_STATUS;
		
		String dicomStatus = REPORT_VALUES.getProperty( status );
		if ( dicomStatus == null ) {
			throw new IMException( "Status value is unrecognized" );
		}

		Report.Participants participants = new Report.Participants();
		
		
		//Results
		if ( uidPair.getProposedStudy() != null ) {
			mod.append( "(0020,000d) UI [" + uidPair.getProposedStudy() + "]\n" ); // Study Instance UID
		}
		else if ( uidPair.getStudy() != null ) {
			mod.append( "(0020,000d) UI [" + uidPair.getStudy() + "]\n" ); // Study Instance UID
		}
		
		if ( params.getProperty( "SPUI" ) != null ) {
			mod.append( "(0008,0018) UI [" + params.getProperty( "SPUI" ) + "]\n" ); // SR SOP Instance UID
		}
		mod.append( "(f215,1031) SQ\n" ); // PBResultsSequence
		mod.append( " (fffe,e000) na # item begin\n" );
//		mod.append( "  (0008,0018) UI [" + sopInstanceUID + "]\n" ); // SOP Instance UID
//		mod.append( "  (4008,0040) SH [" + resID + "]\n" );          // Results ID
		mod.append( "  (4008,0300) ST [FILE:" + impressionFileName + "]\n" );   // Impression
		mod.append( " (fffe,e00d) na # item end\n" );
		mod.append( "(fffe,e0dd) na # sequence end\n" );

		//Interpretation
		mod.append( "(f215,1032) SQ\n" ); // PBInterpretationSequence
		mod.append( " (fffe,e000) na # item begin\n" );
//		mod.append( "  (0008,0018) UI [" + sopInstanceUID + "]\n" ); // SOP Instance UID
		mod.append( "  (4008,0200) SH [" + params.getProperty( "INID", createID() ) + "]\n" );  // Interpretation ID

		addAuthorObserver( participants, params.getProperty( "ITAI" ), params.getProperty( "ITAN" ), params.getProperty( "IRDT" ) );
		addTranscriptionist( participants, params.getProperty( "ITPI" ), params.getProperty( "ITPN" ), params.getProperty( "ITDT" ) );
		
		
		mod.append( "  (4008,010b) ST [FILE:" + textFileName + "]\n" );   // Text
		
		if ( FINAL_STATUS.equals( status ) ) {
			if ( params.getProperty( "IAPN" ) == null ) {
				params.setProperty( "IAPN", params.getProperty( "ITAN", "" ) );
			}

			if ( params.getProperty( "IADT" ) == null ) {
				params.setProperty( "IADT", params.getProperty( "IRDT", "_" ) );
			}

			addApprover( participants, params.getProperty( "IAPI" ), params.getProperty( "IAPN" ), params.getProperty( "IADT" ) );
 		}
		
		mod.append( "(4008,0212) CS [").append( dicomStatus ).append("]\n"); // ISTI: Status ID
		
		StringBuilder createOnlyMod = new StringBuilder( 256 );

		participants.addParticipants( mod, createOnlyMod );

		String normality = params.getProperty( "INIS" );
		if ( imagemed.types.enumeration.IMTypeni.isValidValue( normality ) ) {
			mod.append( "(f215,1006) CS [" + normality + "]\n" ); // PBNormalIndication
		}

		String birads = params.getProperty( "BRDR" );
		if ( imagemed.types.enumeration.IMTypeBRDS.isValidValue( birads ) ) {
			mod.append( "(f215,1007) CS [" + birads + "]\n" ); // BI-RADS Code
		}
		
		
		String locationID = params.getProperty( "ILCI" );
		if ( locationID != null && !locationID.isEmpty() ) { 
			String locationMod = Report.getLocationMod( locationID, false );
			if ( locationMod != null ) {
				createOnlyMod.append( locationMod );
			}
		}

		if ( createOnlyMod.length() > 0 ) {
			mod.append( "(f215,1041) SQ # Create Only Sequence\n" );
			mod.append( " (fffe,e000) na # item begin\n" );
			mod.append( createOnlyMod );
			mod.append( " (fffe,e00d) na # item end\n" );
			mod.append( "(fffe,e0dd) na # sequence end\n" );
		}
		
		mod.append( "  (4008,0210) CS [REPORT]\n" );     // Interpretation Type ID
		mod.append( " (fffe,e00d) na # item end\n" );
		mod.append( "(fffe,e0dd) na # sequence end\n" );


		Integer resultCode = Report.rmiImportReport( null, uidPair.getStudy(), mod.toString(), reportFiles, is.regID, new Boolean( "true".equals( merge ) ) );
		if ( resultCode.intValue() != 0 ) {
			errorMsg = "An error occured while saving data.";
		}
	}
	catch ( IMException imEx ) {
		Helper.ERRORLOG( "importReport", Helper.throwable2String( imEx ) );
		errorMsg = imEx.getMessage();
		if (errorMsg == null)
			errorMsg = "An error occurred while saving data";
	}
	catch ( Throwable th ) {
		Helper.ERRORLOG( "importReport", Helper.throwable2String( th ) );
		errorMsg = "An error occured while saving data:" + th.toString();
	}


	if ( parBack != null ) {
		// Redirect

		// Replace $$USERID$$ with is.regID in parBack
		// to pass it back to Valyon Joe's program (mammo project)

		String nextParamChar = "&";
		if ( parBack.indexOf( "?" ) == -1 )
			nextParamChar = "?";

		parBack = Helper.replace( parBack, "$$USERID$$", is.regID );

		parBack = parBack + nextParamChar + "cResult=" + ( errorMsg == null ? "OK" : "FAILED" );
		parBack = parBack + "&cUserid=" + Helper.URLEncodeUTF8( ( is.regID == null ? "null" : is.regID ) );
		if ( errorMsg != null ) {
			parBack = parBack + "&cError=" + Helper.URLEncodeUTF8( errorMsg );
		}
		response.sendRedirect( parBack );
		return;
	}
	else {
		response.setContentType( "text/plain" );%>#USERID: <%=is.regID%>
<%
		if ( errorMsg != null ) {

%>#RESULT: FAILED
#ERROR: <%=errorMsg%>
<%
		}
		else {

%>#RESULT: OK
<%
		}
	}

	if ( true )
		return;
%>
