<%--
 * Component: iface
 *
 * WEB interface for performing several operations on studies
 *
 *  @param cStudy        Study Instance UID of the study/studies to perform operation on
 *  @param cOper         Operation to be performed
 *
 * Supported operations are:
 *
 *  - delete   : delete study/studies
 *             has no additional parameters
 *
 *  - obsolete : obsoletes study (NOT IMPLEMENTED)
 *             has no additional parameters
 *
 *  - forward  : forwards study
 *             cTarget : list of devices where study/studies will be forwarded
 *
 *  - burn2cd   : burn studies to CD (NOT IMPLEMENTED)
 *
 * $Author$
 * $Date$
 *
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
--%><%@
	page import="imagemed.util.*,	imagemed.exception.*,imagemed.imsession.*,imagemed.jsputil.*, imagemed.dcprocessing.StudyForwarder, java.util.*"

%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%!static Syslog slog = new Syslog("ifaceoper", 0 , IMEnvBase.getEnv("LOG_OPER_FACILITY"));


	/**
	* Deletes studies from the server
	*
	* @param is        Session stuff (login, rights)
	* @param studies   Studies to be deleted from the server
	*
	* @throws IMException if an error occurs
	*/
	void deleteStudies(IMSession is, String studies[], boolean isForced, boolean isImmediate) throws IMException {
	  IMEnv.debugLog("DELETE study: " + studies[0]);
		// Check for delete right
      boolean bDeleteRight = (is.regRights != null && -1 != is.regRights.indexOf("D"));
		if ( !bDeleteRight ) {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "delete" );
			logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "no rights" );
			logEntry.log( slog, Syslog.LOG_ERR );

//			slog.syslog(Syslog.LOG_ERR,
//					"ERROR STUDY delete " + is.regID + " - - - - - - - - - - - no_rights");
			throw new IMException(is.regID + " has no right to delete study.");
		}

		// check if all study is deletable
		String noDelete = "";
		for (int i = 0; i < studies.length ; i++) {
			if ( !OperToolbox.canDeleteStudy(studies[i]))
				noDelete = noDelete + "," + studies[i];
		}

		if (! "".equals(noDelete)) {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "delete" );
			logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "Study/Studies are undeletable" + noDelete.substring(1) );
			logEntry.log( slog, Syslog.LOG_ERR );

			throw new IMException("UNDELETABLE: " + noDelete.substring(1));
		}

		// Delete the studies
		try {
			StudyToolbox.deleteStudies( studies, is.regID, isForced, null, isImmediate );
		}
		catch (Throwable th) {
			throw new IMException("An error occured while removing study! (" + th.getMessage() + ")");
		}
	} // deleteStudies


	/**
	* Forwards studies to target devices
	*
	* @param is        Session stuff (login, rights)
	* @param studies   Studies to be forwarded
	* @param targets   Target devices
	*
	* @throws IMException if an error occurs
	*/
	void forwardStudies(IMSession is, String studies[], String targets[]) throws IMException {

		// Check for forward right
		boolean bForwardRight = (is.regRights != null && -1 != is.regRights.indexOf("F"));
		if (!bForwardRight) {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "forward" );
			logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "no rights" );
			logEntry.log( slog, Syslog.LOG_ERR );

//			slog.syslog(Syslog.LOG_ERR,
//					"ERROR STUDY forward " + is.regID + " - - - - - - - - - - - no_rights");
			throw new IMException(is.regID + " has no right to forward study.");
		}

		// Check if all of the selected studies can be forwarded
		OperToolbox.CheckedStudies checkedStudies = OperToolbox.getForwardableStudies(studies);
		if (checkedStudies.hasBadStudy()) {
			StringBuffer noFwd = new StringBuffer();
			for (Iterator<String> i = checkedStudies.getBadStudies().iterator(); i.hasNext(); ) {
				noFwd.append( "," ).append( i.next() );

				OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "forward" );
				logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
				logEntry.setField( OperLogEntry.FIELD.MESSAGE, "Study/Studies are unforwardable" + noFwd.substring(1) );
				logEntry.log( slog, Syslog.LOG_ERR );

//				slog.syslog(Syslog.LOG_ERR,
//						"ERROR STUDY forward " + is.regID + " - - - - - - - - - - - Study/Studies_are_unforwardable" +
//						noFwd.substring(1));
				throw new IMException("UNFORWARDABLE:" + noFwd.substring(1));
			}
		}

		for (int i=0; i<targets.length; i++) {
			OperToolbox.checkforwardHost(targets[i]);
		}

		for (int i = 0; i < targets.length; i++) {
			StudyForwarder.forwardStudies( null, Arrays.asList(studies), targets[i], is.regID, false, false );
		}
	}

/*
	void burnStudiesToCd(IMSession is, String studies[], String mediaSize) {

		// Check for forward right
		boolean bForwardRight = (is.regRights != null && -1 != is.regRights.indexOf("F"));
		if (!bForwardRight) {
			slog.syslog(Syslog.LOG_ERR,
					"ERROR STUDY burn2cd " + is.regID + " - - - - - - - - - - - no_rights");
			throw new IMException(is.regID + " has no right to forward study.");
		}

		// Check if all of the selected studies can be forwarded
		String[][] res = OperToolbox.getForwardableStudies(studies);
		if (res[1] != null && res[1].length > 0) {
			String noFwd = "";
			for (int i = 0; i < res[1].length; i++) {
				noFwd = noFwd + "," + res[1][i];

				slog.syslog(Syslog.LOG_ERR,
						"ERROR STUDY burn2cd " + is.regID + " - - - - - - - - - - - Study/Studies_are_unforwardable" +
						noFwd.substring(1));
				throw new IMException(UNFORWARDABLE: noFwd.substring(1));
			}
		}

		// Check total size
		long tsize=0;
		String query = "select SUM(SUMSIZE) as s from "+IMEnv.getEnv("STUDYTBL")+" where " + whereGr;
		Statement  sql = IMEnv.getIMDB().createStatement();
		ResultSet result = sql.executeQuery(query);
		if (result.next()) {
			try {
				tsize = Long.parseLong(result.getString("s"));
			}
			catch (NumberFormatException nfe) {
				throw new RuntimeException("oper.jsp: query = \""+query+"\" result = \""+result.getString("s")+"\"");
			}
		}
		result.close();


		// Check total size
		long maxSize = OperToolbox.getMaxCapacity(mediaSize);
		if (tsize > maxSize) {
			slog.syslog(Syslog.LOG_ERR,
					"ERROR STUDY burn2cd " + is.regID + " - - - - - - - - - - - media_size_exceeded" +
					noFwd.substring(1));
			throw new IMException("Media size exceeded");
		}
	}
*/%><%-- Input Parameters --%><%IMSession is = new IMSession(request, response, out); 

	String errorMsg = null;

	try {

		// Check login
		if (!IfaceToolbox.isRegOK(is)) {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "ifaceoper" );
			logEntry.setField( OperLogEntry.FIELD.HOST, request.getRemoteAddr() );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "no rights" );
			logEntry.log( slog, Syslog.LOG_ERR );

//			slog.syslog(Syslog.LOG_ERR,
//					"ERROR STUDY ifaceoper - - - - - " + request.getRemoteAddr() + " - - - - - - no_rights");
			throw new IMException("Performing operation is disabled");
		}

		String studies[] = request.getParameterValues("cStudy");
		if (studies == null) {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "ifaceoper" );
			logEntry.setField( OperLogEntry.FIELD.HOST, request.getRemoteAddr() );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "Missing parameter: Study Instance UID" );
			logEntry.log( slog, Syslog.LOG_ERR );
			
//			slog.syslog(Syslog.LOG_ERR,
//					"ERROR STUDY ifaceoper - - - - - " + request.getRemoteAddr() + " - - - - - - Missing_parameter:_Study_Instance_UID");
			throw new IMException("Missing parameter: Study Instance UID");
		}

		String parForced = request.getParameter("cForce");
		boolean isForced = false;
		if ("true".equals(parForced)) {
			isForced = true;
		}
			
		String parOper = request.getParameter("cOper");
		if ("delete".equals(parOper)) {
			deleteStudies(is, studies, isForced, false);
		}
		else if ("immediate".equals(parOper)) {
			deleteStudies(is, studies, isForced, true);
			
		}
		
/*		else if ("obsolete".equals(parOper)) {

		}
*/
		else if ("forward".equals(parOper)) {
			String targets[] = request.getParameterValues("cTarget");
			if (targets == null) {
				OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "forward" );
				logEntry.setField( OperLogEntry.FIELD.HOST, request.getRemoteAddr() );
				logEntry.setField( OperLogEntry.FIELD.MESSAGE, "Missing parameter: Target Devices" );
				logEntry.log( slog, Syslog.LOG_ERR );

//				slog.syslog(Syslog.LOG_ERR,
//						"ERROR STUDY forward - - - - - " + request.getRemoteAddr() + " - - - - - - Missing_parameter:_Target_Devices");
				throw new IMException("Missing parameter: Target Devices");
			}
			forwardStudies(is, studies, targets);
		}
/*		else if ("burn2cd".equals(parOper)) {
		}
*/
		else {
			OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.STUDY, "ifaceoper" );
			logEntry.setField( OperLogEntry.FIELD.HOST, request.getRemoteAddr() );
			logEntry.setField( OperLogEntry.FIELD.MESSAGE, "Missing or invalid operation" );
			logEntry.log( slog, Syslog.LOG_ERR );

//			slog.syslog(Syslog.LOG_ERR,
//					"ERROR STUDY ifaceoper - - - - - " + request.getRemoteAddr() + " - - - - - - Missing_or_invalid_operation");
			throw new IMException("Missing or invalid operation");
		}
	}
	catch (IMException imEx) {
		errorMsg = imEx.getMessage();
		if (errorMsg == null)
			errorMsg = "An error occured while saving data";
	}
	catch (Throwable th) {
		errorMsg = "An error occured while saving data:" + th.toString();
	}

	response.setContentType("text/plain");%>#USERID: <%=is.regID%>
<%
	if (errorMsg != null) {

%>#RESULT: FAILED
#ERROR: <%=errorMsg%>
<%
	}
	else {

%>#RESULT: OK
<%
	}

	if (true)
		return;

%>
