<%--
 ** Component: iface
 **
 ** Description:
 **  Server config interface (instead of imadmin/dicomsrv.jsp)
 **
 ** @verbatim
 ** Parameters
 **
 **  Operation Parameters:
 **
 **   @param p_op    config operation {'Save and Restart' | 'Restart' | 'checkLicense' |
 **                                    'Change' | 'SaveHL7' | 'Format' }
 **   @param p_op_sr	->alias for p_op = 'Save and Restart'
 **   @param p_op_r		->alias for p_op = 'Restart'
 **   @param p_op_c		->alias for p_op = 'Change'
 **   @param p_op_s		->alias for p_op = 'SaveHL7'
 **   @param p_op_f		->alias for p_op = 'Format'
 **   @param s_op    Change security settings {'Change'}
 **   @param r_op    Change purge settings {'Change'}
 **   @param d_op    Change disk settings {'Change'}
 **   @param i_op    Change institution settings {'Change'}
 **   @param ip_op   Change local IP settings {'Change'}
 **   @param e_op    Change editing defaults ('Change'}
 **
 **     p_op == 'checkLicense' : Forces license checking inmediatelly and picks up new license-file
 **
 **  Operation Specific Parameters
 **
 **     p_op == 'Save and Restart' : Saves settings for DICOM server and restart it
 **
 **  @param port          TCP port the DICOM server listen on
 **  @param portTLS       TLS port the DICOM server listen on
 **  @param pdu           Default PDU size accepted
 **  @param ae            AE_TITLE of the DICOM server
 **  @param arch          Archive AE TITLE
 **  @param ts            Preferred Transfer Syntax
 **  @param recvto        Default receive timeout
 **  @param sendto        Default send timeout
 **  @param cbQRcaseMode  Existence of the parameter means that QR find is case sensitive
 **  @param cbQRemptyMatching  Existence of the parameter means that QR find also returns empty and NULL values
 **
 **    p_op  == 'Change' : Change Secure Mode / Inactivity timeout
 **
 **  @param vSecure       Secure Mode HTTP/HTTPS
 **  @param vTimeout      Inactivity timeout (0 == infinite)
 **
 **    p_op == 'SaveHL7' : Save HL7 settings
 **
 **  @param hl7Toggle         Is HL7 turned on or not
 **  @param hl7Port           HL7 server port to listen on
 **  @param hl7AppID          Application Entity ID of HL7 server
 **  @param hl7FacID          Facility Entity ID of HL7 server
 **  @param hl7SendCompleted  Indicates whether completed order messages should always be sent or not
 **  @param hl7PatientUpdate  Allow applying patient updates to studies newer than patientUpdate days
 **	 @param hl7ApplyTzOffset  Indicates whether hl7 time zone offset is used or not
 **
 **    p_op == 'Restart' : Restart component
 **
 **  @param p_cmp        Component to be restarted
 **
 **    p_op == 'Format'  : Change Date/Time format
 **
 **  @param dateTag0     1st date format tag
 **  @param dateSep1     Separator tag between 1st and 2nd date format tag
 **  @param dateTag2     2nd date format tag
 **  @param dateSep3     Separator tag between 2nd and 3rd date format tag
 **  @param dateTag4     3rd date format tag
 **  @param timeTag      Time format tag
 **
 **    s_op == 'Change' : Change Set curl / lock timeout
 **
 **  @param curlTimeout  Curl Timeout
 **  @param lTimeout     Lock Timeout
 **
 **    r_op == 'Change' : Change purge settings
 **
 **  @param order        Purge settings for orders
 **  @param final        Purge settings for final studies
 **  @param study        Purge settings for any study
 **
 **    d_op == 'Change' : Change disk settings
 **
 **  @param freelimit    Minimal free disk space (percent)
 **  @param mincache     Minimal cache size (percent)
 **
 **    i_op == 'Change' : Change institution info
 **
 **  @param institution  Institution information
 **
 **    ip_op == 'Change' : Change local IP Configuration
 **
 **  @param localIPs     Local IP address mask(s)
 **
 **    e_op == 'Change' : Change editing defaults
 **
 **  @param editMode          default edit mode
 **  @param correctionMode    correction mode
 **
 ** @endverbatim
 **
 ** $Author$
 ** $Date$
 **
 *
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
--%><%@
	page errorPage ="../main/exceptionHandler.jsp"

%><%@
	page import="imagemed.util.*, imagemed.imsession.*, imagemed.http.tabs.*, imagemed.exception.*,
			imagemed.jsputil.*,imagemed.types.*,imagemed.types.string.*,imagemed.http.*"

%><%IMSession is = new IMSession(request, response, out);
try{
	// ----- Input Parameter(s) -----

	if (!IfaceToolbox.isRegOK(is))
		throw new IMIfaceException(is,"Has no registration info for this IP.");

	if (!IMAdminToolbox.bAdminRight(is))
	  throw new IMIfaceException(is,"Has no Admin right for this operation.");

	boolean isRestricted = is.regRights.indexOf("@") == -1;
	Syslog slog = new Syslog("iface/serverOper", 0 , IMEnvBase.getEnv("LOG_OPER_FACILITY"));

//	out.println("<!--"+Helper.request2String(request)+"-->");

	String parPOp;    // p_op
	                  // ts, pdu, ae [, port]
	                  // recvto, sendto
	                  // vSecure, vTimeout, checkLicense
	String parSOp;    // s_op : saving system properties
	                  // lTimeout
	String parPCmp;   // p_cmp // if p_op == Restart
	String parROp;    // r_op
	                  // other, purge
	String parIOp;    // i_op
	                  // institution
	String parIPOp;   // ip_op
	                  // local IPs
	String parDOp;    // d_op
	                  // disk usage option
	String parEOp;    // e_op
	                  // default Edit / Merge settings

	parPOp   = is.request.getParameter("p_op");
	parPCmp  = is.request.getParameter("p_cmp");
	parROp   = is.request.getParameter("r_op");
	parIOp   = is.request.getParameter("i_op");
	parIPOp  = is.request.getParameter("ip_op");
	parDOp   = is.request.getParameter("d_op");
	parSOp   = is.request.getParameter("s_op");
	parEOp   = is.request.getParameter("e_op");

	IMProperties prpSystem = DicomSrvToolbox.readSystemRec();

	//out.println("<!--"+Helper.request2String(request)+"-->");
	if ( "checkLicense".equals(parPOp) ) {
		final boolean forceCheck = true;
		TabBean tb = new TabBean();
		tb.writeHeader( is, "White", forceCheck, false );
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	}

/*Obsolete?
	if ("change".equals(is.request.getParameter("p_plugins"))) {
		PluginToolbox.processEditorForm(new IMRequest(request));
	}
*/
	if ("change".equals(is.request.getParameter("p_main_plugins"))) {
		PluginToolbox.processMainPluginForm(new IMRequest(request));
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	}

	/*
	 *  Dicom server config
	 */

	if (parPOp != null) {
		if (parPOp.equals("Save and restart")) {
			if (isRestricted) {
			  throw new IMIfaceException(is,"Has no Support right for this operation.");
			}

			try {
				DicomSrvToolbox.saveConfQR(request, is.regID, slog );
			}
			catch (IMException imEx) {
				throw new IMIfaceException(is, imEx.getMessage());
			}

			// *** force restart ***
			parPOp = "Restart";
			parPCmp = "dicom";
		} // Save and restart

		if ( "SaveHL7".equals(parPOp) ) {
//			if (isRestricted) {
//			  throw new IMIfaceException(is,"Has no Support right for this operation.");
//			}
			String hl7Toggle = is.request.getParameter("hl7Toggle");
			String hl7Port = is.request.getParameter("hl7Port");
			String localAppID = is.request.getParameter("hl7AppID");
			String localFacID = is.request.getParameter("hl7FacID");
			String sendCompleted = is.request.getParameter( "hl7SendCompleted" );
			String patientUpdateDays = is.request.getParameter( "hl7PatientUpdate" );
			String applyTzOffset = is.request.getParameter("hl7ApplyTzOffset");

			DicomSrvToolbox.saveHL7Config( hl7Toggle, hl7Port, localAppID, localFacID, sendCompleted, patientUpdateDays, applyTzOffset );

			try {
				imagemed.dotcom.DotcomConfig.reCollect( is.out, null );
			}
			catch (Throwable th) {
				throw new IMIfaceException( is, "Could not recollect dotcom configuration");
			}

			
			// *** force restart ***
			parPOp = "Restart";
			parPCmp = "hl7";
		} // SaveHL7

	/*
	 *  Component restart
	 */

		if (parPOp.equals("Restart")) {
			DicomSrvToolbox.restart(parPCmp);
			is.out.println("#USERID: "+is.regID);
			is.out.println("#RESULT: OK");
			return;
		} // Restart

		if (parPOp.equals("Change")) {
			String secure  = is.request.getParameter("vSecure");
			String timeout = is.request.getParameter("vTimeout");
			DicomSrvToolbox.saveSecurityRec(secure, timeout);
			is.out.println("#USERID: "+is.regID);
			is.out.println("#RESULT: OK");
			return;
		} // Change
	} // parOp != null

	// Saving system properties

	if ("Change".equals(parSOp)) {
		String strLockTimeout = request.getParameter("lTimeout");
		String strCurlTimeout = request.getParameter("curlTimeout");
		if (isRestricted) {
			strCurlTimeout = prpSystem.getProperty("CURL_TIMEOUT");
		}
		DicomSrvToolbox.saveSystemRec(strLockTimeout, strCurlTimeout);
		prpSystem = DicomSrvToolbox.readSystemRec();
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	} // SOp == Change

	if ("true".equals(is.request.getParameter("cNameFormat"))) {
		IMTypePN type = (IMTypePN) IMType.getTypeFor("PN");
		String[] listFormat = IMTypePN.getNameFormat(IMTypePN.LISTNAMEFORMAT);
		String[] defaultFormat = IMTypePN.getNameFormat(IMTypePN.DEFAULTNAMEFORMAT);
		for (int i = 0; i < 10; i++) {
			listFormat[i] = is.request.getParameter("lpart" + i);
			if (listFormat[i] == null) {
				listFormat[i] = "";
			}
			defaultFormat[i] = is.request.getParameter("dpart" + i);
			if (defaultFormat[i] == null) {
				defaultFormat[i] = "";
			}
		}
		type.setNameFormat(IMTypePN.LISTNAMEFORMAT, listFormat);
		type.setNameFormat(IMTypePN.DEFAULTNAMEFORMAT, defaultFormat);
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	}

	if ("Format".equals(parPOp)) { // date format changed
		String strFormat = Helper.readTextFile( IMEnvBase.getEnv("CONF") + "/dateformat.cfg" );
		String[] dFormat = imagemed.types.date.IMTypeDA.divideFormatStr(strFormat);

		String tFormat = Helper.readTextFile( IMEnvBase.getEnv("CONF") + "/timeformat.cfg" );
		if (tFormat==null) {
			tFormat="hh:mm:ss";
		}
		if (tFormat.length() < 5 ) {
			tFormat="";
		}
		else if (tFormat.length() < 8) {
			tFormat="hh:mm";
		}
		else {
			tFormat="hh:mm:ss";
		}
		dFormat[0] = is.request.getParameter("dateTag0");
		dFormat[1] = is.request.getParameter("dateSep1");
		dFormat[2] = is.request.getParameter("dateTag2");
		dFormat[3] = is.request.getParameter("dateSep3");
		dFormat[4] = is.request.getParameter("dateTag4");

		if (dFormat[0].equals(" ")) {
			dFormat[0] = "";
		}
		if (dFormat[2].equals(" ")) {
			dFormat[2] = "";
		}
		if (dFormat[4].equals(" ")) {
			dFormat[4] = "";
		}
		Helper.writeTextFile( IMEnvBase.getEnv("CONF") + "/dateformat.cfg",
		dFormat[0]+dFormat[1]+dFormat[2]+dFormat[3]+dFormat[4] );

		tFormat=is.request.getParameter("timeTag");

		Helper.writeTextFile( IMEnvBase.getEnv("CONF") + "/timeformat.cfg",	tFormat);
		imagemed.types.date.IMTypeDA.readFormat();
		imagemed.types.date.IMTypeTM.readFormat();
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	} // POp == Format (date format changed)

	if ("Change".equals(parEOp)) {
		if (isRestricted) {
		  throw new IMIfaceException(is,"Has no Support right for this operation.");
		}
		String editMode = request.getParameter("editMode");
		String correctionMode = request.getParameter("correctionMode");
		DicomSrvToolbox.saveEditCfg(editMode, correctionMode, null,null);
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	} // EOp == Change


	if (parROp != null && parROp.equals("Change")) {
		if (isRestricted) {
		  throw new IMIfaceException(is,"Has no Support right for this operation.");
		}
		String finl  = is.request.getParameter("final");
		String study = is.request.getParameter("study");
		String order = is.request.getParameter("order");
		String limit = is.request.getParameter("limit");
		DicomSrvToolbox.saveDailyRec(finl, study, order, limit);
		
		OperLogEntry logEntry = new OperLogEntry( OperLogEntry.LEVEL.ADMIN, "serverregular" );
		logEntry.setField( OperLogEntry.FIELD.USER, is.regID );
		logEntry.setField( OperLogEntry.FIELD.SERVER_REGULAR_PURGE_STUDIES, study );
		logEntry.setField( OperLogEntry.FIELD.SERVER_REGULAR_PURGE_FINALS, finl );
		logEntry.setField( OperLogEntry.FIELD.SERVER_REGULAR_PURGE_ORDERS, order );
		logEntry.setField( OperLogEntry.FIELD.SERVER_FILTERED_LIMIT, limit );
		logEntry.log( slog, Syslog.LOG_INFO );

//		slog.syslog(Syslog.LOG_INFO, "INFO ADMIN serverregular "+ is.regID +" - - - - - - - - - - - - - - - - - - - - - - - - - "+ Syslog.logValue(study) +" "+ Syslog.logValue(finl) +" "+ Syslog.logValue(order) +" "+ Syslog.logValue(limit) +" - - - - -");

		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	} // ROp == Change


	// Saving disk limit settings
	if ("Change".equals(parDOp)) {
		if (isRestricted) {
		  throw new IMIfaceException(is,"Has no Support right for this operation.");
		}
		String parFreeLimit = is.request.getParameter("freelimit");
		String parMinCache  = is.request.getParameter("mincache");
		DicomSrvToolbox.saveLimitConf(parFreeLimit, parMinCache);
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	} // DOp == Change


	if ("Change".equals(parIOp)) {
		DicomSrvToolbox.saveInstitution(is.request.getParameter("institution"));
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	}

	if ("Change".equals(parIPOp)) {
		DicomSrvToolbox.saveLocalIPsCfg(is.request.getParameter("localIPs"));
		is.out.println("#USERID: "+is.regID);
		is.out.println("#RESULT: OK");
		return;
	}
} catch (IMIfaceException imi) {
	is.out.print( imi.getFormattedMessage() );
	return;
} catch (Throwable t) {
	IMIfaceException imi = new IMIfaceException(is,t.getMessage());
	is.out.print( imi.getFormattedMessage() );
	return;
}%>
