<%--
 * Component: iface
 *
 * Description:
 *    Allow opening a study using StudyUID or Accession Number
 * 
 * If there are more than one studies for an accession number, an error response will be returned.
 *
 * @params:
 *
 *   ACCN   accession number of the study to be opened.
 *	 PAID		patientID to be used with ACCN to identify study
 * 
 *   SYUI   study instance UID of the study to be opened (if this is present, ACCN and PAID is ignored)
 *
 *	 cSeries	Series Instance UID, if this parameter is given, the webviewer will open up the given serie
 *
 *	 cImageNumber	number of frame in the given series to be opened 
 *
 *   cMultipleMode    how to handle multiple matches on accession numbers
 *                    possible values are
 *                    normal :  return an error message (default)
 *                    list   :  popup worklist with matching studies
 *
 *   cUser    user id
 *   cPasswd  password
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 *
 --%><%@page import="imagemed.jsputil.IfaceToolbox,
 		imagemed.util.Helper,
 		java.util.LinkedHashMap,
 		imagemed.tables.SQLResult,
 		java.util.Properties,
 		imagemed.tables.TableParams,
 		imagemed.tables.TableDescriptor,
 		imagemed.imtables.IMFieldManager,
 		java.util.StringTokenizer,
 		java.util.Vector,
 		imagemed.imsession.IMSession"
 
 %><%!

	static final int MULTIPLE_MODE_NORMAL = 0;
	static final int MULTIPLE_MODE_LIST = 1;

%><%
	IMSession is = new IMSession( request, response, out );
	String parAccno = request.getParameter( "ACCN" );
	String parStudy = request.getParameter( "SYUI" );
	String parMultipleMode = request.getParameter( "cMultipleMode" );
	
	int multipleMode = MULTIPLE_MODE_NORMAL;
	if ( "list".equals( parMultipleMode ) ) {
		multipleMode = MULTIPLE_MODE_LIST;
	}

	if ( parAccno == null && parStudy == null ) {
		out.println( "ERROR: Missing Parameter" );
		return;
	}

	is.sessionVariables.setProperty("httpIface", "true");

	if ( parStudy == null ) { // in this case parAccno != null
		String fields[] = { "SYUI", "PAID" };
		IMFieldManager fm = new IMFieldManager( "Dcfields", "Dctypes", "Dcstudy" );

		TableDescriptor tdesc = new TableDescriptor();
		TableParams tparams = new TableParams();
		tparams.setDefaultFields( fields );

		Properties filters = new Properties();
		filters.setProperty( "ACCN", parAccno );
		filters.setProperty( "FFLG", "\"\"" );
		filters.setProperty( "STDE", "no" );
		
		String parPatient = request.getParameter( "PAID" );
		if ( parPatient != null ) {
			filters.setProperty( "PAID", parPatient );
		}
		
		tparams.setFilters( filters );
		tparams.setDefaultSort( new String[]{ "d_DATE", "a_SYUI" } );
		tparams.setDefaultPageSize( -1 );

		tdesc.setTableParams( tparams );
		tdesc.setFieldManager( fm );
		SQLResult result = new SQLResult( tdesc );

		while ( result.next() ) {
			String study = result.getValue( "SYUI" );
			
			if ( parStudy != null ) {
				// More than one studies for accession number
				if ( multipleMode == MULTIPLE_MODE_NORMAL ) {
					// Normal behavior: return error message
					out.println( "ERROR: There are more than one studies for accession number " + parAccno );
					result.close();
					return;
				}
				else if ( multipleMode == MULTIPLE_MODE_LIST ) {
					// Popup worklist
					StringBuffer redirectParams = new StringBuffer();
					if ( is._sid != null ) {
						redirectParams.append( "&sessionid=" ).append( Helper.URLEncodeUTF8( is._sid ) );
					}
					if ( parAccno != null ) {
						redirectParams.append( "&qf_ACCN=" ).append( Helper.URLEncodeUTF8( parAccno ) );
					}
					if ( parStudy != null ) {
						redirectParams.append( "&qf_SYUI=" ).append( Helper.URLEncodeUTF8( parStudy ) );
					}
					if ( parPatient != null ) {
						redirectParams.append( "&qf_PAID=" ).append( Helper.URLEncodeUTF8( parPatient ) );
					}

					IfaceToolbox.loginAndRedirect( is, is.httpx_host + "/cases/index.jsp?" + redirectParams.substring( 1 ) );
					return;
				}
			}

			parStudy = study;
		}
		result.close();
	}

	if ( parStudy == null ) {
		out.println( "ERROR: No study found" );
		return;
	}
	
	String loc = is.httpx_host + "/EPVWeb/EPVWeb.jsp?cStudy=" + parStudy;

	String sessionID =  request.getParameter( "sessionid" );
	if( sessionID != null ) {
		loc += "&sessionid=" + sessionID;
	}
	String parSeries = request.getParameter( "cSeries" );
	if( parSeries != null ) {
		loc += "&cSeries=" + parSeries;
		String parNum = request.getParameter( "cImageNumber" );
		if( parNum != null ) {
			loc += "&cImageNumber=" + parNum;
		}
	}
	
	IfaceToolbox.loginAndRedirect( is, loc + "#studyview" );

	if ( true ) {
		// To avoid sending any other character on the connection after redirect
		return;
	}
%>